package com.example.fuzzControll.controller.dataController;

import com.example.fuzzControll.domain.bo.CmdStartParams;
import com.example.fuzzControll.domain.bo.TestEntity;
import com.example.fuzzControll.domain.po.TestAndParams;
import com.example.fuzzControll.domain.vo.AjaxResult;
import com.example.fuzzControll.exception.testException.CmdException;
import com.example.fuzzControll.service.SeedFileService;
import com.example.fuzzControll.service.TestMissionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 测试信息控制栏
 */
@Slf4j
@RestController
@RequestMapping("/testmission")
public class TestMissionController {
    @Autowired
    TestMissionService testMissionService;

    /**
     *
     * @param id 测试id用来获取参数
     * @param
     */
    @RequestMapping(value = "/startmission", method = RequestMethod.GET)
    public AjaxResult startmission(@RequestParam int id) {
        try {
            testMissionService.startMission(id);
        } catch (Exception e) {
            return AjaxResult.error("测试启动失败！");
        }
        return AjaxResult.success("测试启动成功！");
    }
    /**
     * 测试信息获取
     */
    @RequestMapping(value = "/getAll", method = RequestMethod.GET)
    public AjaxResult getTestAndParamsList() {
        List<TestAndParams> testAndParamsList = null;
        try {
            testAndParamsList = testMissionService.getTestAndParamsList();
        } catch (Exception e) {
            return AjaxResult.error("测试信息获取失败！");
        }
        return AjaxResult.success(testAndParamsList);
    }

    /**
     * 测试信息删除
     */
    @RequestMapping(value = "/deleteById", method = RequestMethod.GET)
    public AjaxResult deleteById(@RequestParam int id) {
        boolean flag = false;
        try {
            flag = testMissionService.deleteById(id);
            if (!flag) {
                throw new RuntimeException();
            }
        } catch (Exception e) {
            return AjaxResult.error("删除失败！");
        }
        return AjaxResult.success("删除成功！");
    }

    /**
     * 新建测试
     */
    @RequestMapping(value = "/insert", method = RequestMethod.GET)
    public AjaxResult insert(@RequestParam String testname) {
        TestAndParams testAndParams = new TestAndParams(testname);
        try {
            testMissionService.insert(testAndParams);
        } catch (Exception e) {
            return AjaxResult.error("新建失败！");
        }
        return AjaxResult.success("新建成功！");
    }

    /**
     * 给空白测试配置参数-aflnet
     */
    @RequestMapping(value = "/insertWithAflnet/{id}", method = RequestMethod.POST)
    public AjaxResult insertParmasInTestAflnet(@PathVariable("id")int id,@RequestBody final CmdStartParams cmdStartParams) {
        try {
            testMissionService.insertParmasInTestAflnet(id,cmdStartParams);
        } catch (Exception e) {
            return AjaxResult.error("参数配置失败！");
        }
        return AjaxResult.success("参数配置成功！");
    }

    /**
     * 给空白测试配置参数-kitty
     */
    @RequestMapping(value = "/insertWithkitty/{id}", method = RequestMethod.POST)
    public AjaxResult insertParmasInTestKitty(@PathVariable("id")int id,@RequestBody TestEntity testEntity) {
        try {
            testMissionService.insertParmasInTestKitty(id,testEntity);
        } catch (Exception e) {
            return AjaxResult.error("参数配置失败！");
        }
        return AjaxResult.success("参数配置成功！");
    }
    /**
     * 测试名称修改
     */
    @RequestMapping(value = "/editTestName", method = RequestMethod.GET)
    public AjaxResult editTestName(@RequestParam String testName,@RequestParam int id) {
        try {
            testMissionService.editTestName(testName,id);
        } catch (Exception e) {
            return AjaxResult.error("修改失败！");
        }
        return AjaxResult.success("修改成功！");
    }
}
