package com.example.fuzzControll.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.fuzzControll.constents.MissionStateEnum;
import com.example.fuzzControll.constents.TableClassEnum;
import com.example.fuzzControll.domain.bo.CmdStartParams;
import com.example.fuzzControll.domain.bo.FuzzParams;
import com.example.fuzzControll.domain.bo.TestEntity;
import com.example.fuzzControll.domain.po.MissionInfo;
import com.example.fuzzControll.domain.po.Test;
import com.example.fuzzControll.domain.po.TestAndParams;
import com.example.fuzzControll.domain.vo.AjaxResult;
import com.example.fuzzControll.exception.mysqlException.MysqlException;
import com.example.fuzzControll.exception.threadException.MyExceptionHandler;
import com.example.fuzzControll.mapper.FuzzParamsMapper;
import com.example.fuzzControll.mapper.MissionInfoMapper;
import com.example.fuzzControll.mapper.TestMapper;
import com.example.fuzzControll.service.TestMissionService;
import com.example.fuzzControll.service.TestService;
import com.example.fuzzControll.tools.component.StringUtils;
import com.example.fuzzControll.tools.system.GlobalClass;
import com.example.fuzzControll.tools.system.SystemRunningParams;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service("TestMissionServiceImpl")
public class TestMissionServiceImpl implements TestMissionService {
    @Autowired
    TestMapper testMapper;
    @Autowired
    FuzzParamsMapper fuzzParamsMapper;
    @Autowired
    MissionInfoMapper missionInfoMapper;
    @Autowired
    TestService service;

    @Override
    public List<TestAndParams> getTestAndParamsList() {
        List<TestAndParams> testParamsRelevantList = new ArrayList();
        List<TestAndParams> testParamsListEmpty = new ArrayList();
        try {
            /*查询关联的数据*/
            testParamsRelevantList = testMapper.getTestAndParamsList();
            /*查询空白数据，组合起来是全数据*/
            testParamsListEmpty = testMapper.getTestList().stream().map(test -> new TestAndParams(test.getId(), test.getFuzzParamsId(), test.getCreateTime()
                    , test.getUpdateTime(), test.getMissionIds(), test.getTestName(), "", "")).collect(Collectors.toList());
            testParamsRelevantList.addAll(testParamsListEmpty);
        } catch (Exception e) {
            log.error("Failed to get test and params list:{}", e.getMessage());
            throw new RuntimeException(e);
        }
        if (testParamsRelevantList == null || testParamsRelevantList.size() == 0) {
            log.warn("TestAndParams is null!");
        }
        return testParamsRelevantList;
    }

    @Override
    public boolean deleteById(int id) {
        boolean flag = false;
        try {
            Test test = testMapper.getTestById(id);
            /*非空白测试级联删除*/
            if (!(test.getFuzzParamsId() == 0)) {
                flag = testMapper.deleteById(id);
            } else {
                /*空白测试删除*/
                flag = testMapper.deleteEmptyById(id);
            }
        } catch (Exception e) {
            log.error("Delete error:{}", e.getMessage());
            throw new RuntimeException(e);
        }
        return flag;
    }


    @Override
    public void insert(TestAndParams testAndParams) {
        Test test = new Test(new Date(), new Date(), testAndParams.getTestName());
        testMapper.insert(test);
    }

    @Transactional(rollbackFor = MysqlException.class)
    @Override
    public void insertParmasInTestAflnet(int id, CmdStartParams cmdStartParams) {
        try {
            /*判断test是否存有paramsId,没数据插入有数据就改变*/
            Test test = testMapper.getTestById(id);
            int fuzzParamsId;
            if (test.getFuzzParamsId() == 0) {//为0则插入id
                FuzzParams fuzzParams = new FuzzParams(JSON.toJSONString(cmdStartParams), cmdStartParams.getProtopcol());
                fuzzParamsMapper.insertFuzzParamsReturnId(fuzzParams);
                fuzzParamsId = fuzzParams.getId();
                test.setFuzzParamsId(fuzzParamsId);
            } else {//有则更新数据
                FuzzParams fuzzParams = new FuzzParams(test.getFuzzParamsId(), JSON.toJSONString(cmdStartParams), cmdStartParams.getProtopcol());
                if (!fuzzParamsMapper.editFuzzParams(fuzzParams.getId(), fuzzParams.getParams(), fuzzParams.getProtocol())) {
                    throw new RuntimeException("EditFuzzParams failed!");
                }
            }
            /*更新修改时间*/
            test.setUpdateTime(new Date());
            testMapper.edit(test);
        } catch (Exception e) {
            log.error("Failed to insert:[{}]", e.getMessage());
            throw new MysqlException("");
        }
    }

    @Transactional(rollbackFor = MysqlException.class)
    @Override
    public void insertParmasInTestKitty(int id, TestEntity testEntity) {
        try {
            /*判断test是否存有paramsId,没数据插入有数据就改变*/
            Test test = testMapper.getTestById(id);
            int fuzzParamsId;
            if (test.getFuzzParamsId() == 0) {//为0则插入id
                FuzzParams fuzzParams = new FuzzParams(JSON.toJSONString(testEntity.getParamJson()), testEntity.getTestClassName());
                fuzzParamsMapper.insertFuzzParamsReturnId(fuzzParams);
                fuzzParamsId = fuzzParams.getId();
                test.setFuzzParamsId(fuzzParamsId);
            } else {//有则更新数据
                FuzzParams fuzzParams = new FuzzParams(JSON.toJSONString(testEntity.getParamJson()), testEntity.getTestClassName());
                if (!fuzzParamsMapper.editFuzzParams(fuzzParams.getId(), fuzzParams.getParams(), fuzzParams.getProtocol())) {
                    throw new RuntimeException("EditFuzzParams failed!");
                }
            }
            /*更新修改时间*/
            test.setUpdateTime(new Date());
            testMapper.edit(test);
        } catch (Exception e) {
            log.error("Failed to insert:[{}]", e.getMessage());
            throw new MysqlException("");
        }
    }

    @Override
    public boolean editTestName(String testName, int id) {
        boolean flag = false;
        try {
            flag = testMapper.editTestName(testName, id);
        } catch (Exception e) {
            log.error("EditTestName failed!:[{}]", e.getMessage());
            throw new RuntimeException(e);
        }
        return flag;
    }

    @Transactional(rollbackFor = RuntimeException.class)
    @Override
    public void startMission(int id) {
        Boolean isAflnet = false;
        Test test = testMapper.getTestById(id);
        if (test.getFuzzParamsId() == 0) {
            log.error("This is a empty test!");
            throw new RuntimeException();
        }
        FuzzParams fuzzParams = null;
        try {
            fuzzParams = fuzzParamsMapper.getFuzzParamsById(test.getFuzzParamsId());
        } catch (Exception e) {
            log.error("Get fuzzParams error!:[{}]", e.getMessage());
            throw new RuntimeException(e);
        }
        /*生成任务，并插入任务数据，回填测试生成的任务字段*/
        String protocol = fuzzParams.getProtocol();
        MissionInfo missionInfo;
        if (Arrays.asList(SystemRunningParams.AflnetProtocol).contains(protocol)) {
            missionInfo = new MissionInfo(TableClassEnum.AFLNET.getTableId(), new Date(), protocol, MissionStateEnum.STARTING.getStateCode(), 0L);
            isAflnet = true;
        } else if (Arrays.asList(SystemRunningParams.KittyProtocol).contains(protocol)) {
            missionInfo = new MissionInfo(TableClassEnum.KITTY_RESULT.getTableId(), new Date(), protocol, MissionStateEnum.STARTING.getStateCode(), 0L);
            isAflnet = false;
        } else {
            log.error("Unknown protocol!");
            throw new RuntimeException();
        }
        Boolean flag = false;
        try {
            flag = missionInfoMapper.insertMission(missionInfo);
        } catch (Exception e) {
            log.error("Error inserting mission:[{}]", e.getMessage());
            throw new RuntimeException(e);
        }
        if (!flag) {
            log.error("Insert mission error!");
            throw new RuntimeException();
        }
        try {
            String missionIds = StringUtils.getArrayFormatString(test.getMissionIds(), String.valueOf(missionInfo.getId()));
            test.setMissionIds(missionIds);
            testMapper.update(test);
        } catch (Exception e) {
            log.error("Update test error:[{}]", e.getMessage());
            throw new RuntimeException(e);
        }
        /*运行程序*/
        try {
            if (isAflnet) {// 启动alfnet
                aflnetMissionStart(fuzzParams,missionInfo);
            } else {

            }
        } catch (Exception e) {
            log.error("Mission start error:[{}]", e.getMessage());
            throw new RuntimeException(e);
        }
    }

    /**
     * 启动aflnet任务
     */
    private void aflnetMissionStart(FuzzParams fuzzParams,MissionInfo missionInfo) {
        CmdStartParams cmdStartParams = (CmdStartParams)JSONObject.parseObject(fuzzParams.getParams(), CmdStartParams.class);
        try {
            SystemRunningParams.aflnetData.put("missionName", cmdStartParams.getProtopcol());
            Thread subThread = new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        missionInfo.setState(MissionStateEnum.RUNNING.getStateCode());
                        missionInfoMapper.updateMission(missionInfo);
                    } catch (Exception e) {
                        log.error("Error updating mission:[{}]",e.getMessage());
                        throw new RuntimeException(e);
                    }
                    service.testStart(cmdStartParams,missionInfo);
                }
            });
            SystemRunningParams.ThreadState.put(subThread, "start");
            subThread.setUncaughtExceptionHandler(new MyExceptionHandler());
            subThread.start();
//            subThread.join();
            Thread.sleep(1000 * 10);//暂停3s,让系统运行至指令完全运行 todo跟据系统运行设定时间
            if ("error".equals(SystemRunningParams.ThreadState.get(subThread))) {//使用join串行就无法停止
                throw new Exception();
            }
        } catch (Exception e) {
            throw new RuntimeException();
        }
    }
}
